local ADDON_NAME, Core = ...

--<
local string = string
local utils = Core.utils
-->

local frameCache = {}
Core.orderedTabs = {}


Core.clientTextures = {
	["App"] = "Interface/CHATFRAME/UI-ChatIcon-Battlenet",
	["D3"] = "Interface/CHATFRAME/UI-ChatIcon-D3",
	["Hero"] = "Interface/CHATFRAME/UI-ChatIcon-HotS",
	["S1"] = "Interface/CHATFRAME/UI-ChatIcon-SC",
	["S2"] = "Interface/CHATFRAME/UI-ChatIcon-SC2",
	["WoW"] = "Interface/CHATFRAME/UI-ChatIcon-WoW",
	["WTCG"] = "Interface/CHATFRAME/UI-ChatIcon-WTCG",
	["Pro"] = "Interface/CHATFRAME/UI-ChatIcon-Overwatch",
	["DST2"] = "Interface/CHATFRAME/UI-ChatIcon-Destiny2",
	["VIPR"] = "Interface/CHATFRAME/UI-ChatIcon-CallOfDutyBlackOps4",
	["W3"] = "Interface/CHATFRAME/UI-ChatIcon-Warcraft3Reforged",
	["ODIN"] = "Interface/CHATFRAME/UI-ChatIcon-CallOfDutyMWicon",
}


Core.clientStrings = {
	["App"] = "In App",
	["D3"] = "Playing D3",
	["Hero"] = "Playing HotS",
	["S1"] = "Playing SC1",
	["S2"] = "Playing SC2",
	["WTCG"] = "In Hearthstone",
	["Pro"] = "Playing OW",
	["DST2"] = "In Destiny2",
	["VIPR"] = "Playing CoD",
	["W3"] = "Playing W3:R",
	["ODIN"] = "Playing CoD",
}


function Core.AppendTab (f)
	if #Core.orderedTabs == 0 then
		f:SetPoint("TOP", 0, -4)
		f.tabIndex = 1
	else 
		f:SetPoint("TOP", 0, -(#Core.orderedTabs * Core.TAB_HEIGHT + 5))
		f.tabIndex = #Core.orderedTabs
	end
	
	if not f:IsShown() then f:Show() end
end


function Core.Tab_ReanchorAll ()
	if #Core.orderedTabs >= 1 then
		for i, t in ipairs(Core.orderedTabs) do
			t:ClearAllPoints()
			
			if i == 1 then
				t:SetPoint("TOP", 0, -4)
				
			else
				t:SetPoint("TOP", 0, -(i-1) * Core.TAB_HEIGHT - 5)
			end
			
			t.tabIndex = i
		end
	end
	
	Core.UI:PseudoScrollUpdate()
end


function Core.CloseTab (f)
	for i, tab in ipairs(Core.orderedTabs) do
		if tab == f then
			table.remove(Core.orderedTabs, i)
			f:Hide()
			f.tabIndex = 0
			break
		end
	end
	
	f:ClearAllPoints()
	table.insert(frameCache, f)
	
	Core.Tab_ReanchorAll()
end


function Core.BNetDisplayUpdate (tab, charName, client, isOnline)
	local fallback

	if isOnline then 
		fallback = "|cFFff794dAway|r"
	else
		fallback = "|cFFb3b3b3Offline|r"
	end
	
	tab.portrait:SetTexture(Core.clientTextures[client] or Core.clientTextures["App"])
	tab.infoString:SetText(charName or Core.clientStrings[client] or fallback)
end


-------------------------------------------------------------------
local function acquireTabFrame ()
-------------------------------------------------------------------
	if #frameCache > 0 then
		return table.remove(frameCache)
	end

	local frame = CreateFrame("Frame", nil, Core.UI.tabWindow.manager)
	frame:SetSize(100, Core.TAB_HEIGHT)
	frame:EnableMouse(true)
	frame:SetClipsChildren(true)
	frame:SetBackdrop({
		edgeFile="Interface/Tooltips/UI-Tooltip-Border",
		edgeSize=8,
	})
	
	frame.BG = frame:CreateTexture(nil, "BACKGROUND", nil, 1)
	frame.BG:SetColorTexture(1,1,1)
	frame.BG:SetGradient("Horizontal", 0,0,0, .2,.2,.38)
	frame.BG:SetPoint("CENTER")
	frame.BG:SetSize(frame:GetSize())
	
	frame.portrait = frame:CreateTexture(nil, "BACKGROUND", nil, 2)
	frame.portrait:SetPoint("RIGHT", 2, 0)
	frame.portrait:SetSize(48, 48)
	frame.portrait:SetAlpha(0.6)
	
	frame.sessionName = frame:CreateFontString(nil)
	frame.sessionName:SetFont("Fonts/FRIZQT__.ttf", 14, "OUTLINE")
	frame.sessionName:SetPoint("TOPLEFT", 3, -4)
	
	frame.infoString = frame:CreateFontString(nil, "MEDIUM")
	frame.infoString:SetFont("Fonts/FRIZQT__.ttf", 10, "OUTLINE")
	frame.infoString:SetPoint("BOTTOMLEFT", 3, 5)
	
	frame.highlight = frame:CreateTexture(nil, "HIGHLIGHT")
	frame.highlight:SetTexture("Interface/BUTTONS/UI-Listbox-Highlight2")
	frame.highlight:SetAlpha(0.2)
	frame.highlight:SetBlendMode("ADD")
	frame.highlight:SetPoint("CENTER")
	frame.highlight:SetSize(frame:GetSize())
	
	local glow = CreateFrame("Frame", nil, frame)
	glow.gtexture = glow:CreateTexture(nil, "OVERLAY")
	glow.gtexture:SetTexture("Interface/Scenarios/Objective-Lineglow")
	glow.gtexture:SetPoint("TOP", frame)
	glow.gtexture:SetSize(frame:GetWidth(), 36)	
	glow:Hide()

	glow.animation = glow:CreateAnimationGroup()
	glow.gpulse = glow.animation:CreateAnimation("Alpha")
	glow.gpulse:SetDuration(0.7)
	glow.gpulse:SetFromAlpha(0)
	glow.gpulse:SetToAlpha(0.7)	
	glow.animation:SetLooping("Bounce")
	
	frame.glow = glow
	
	return frame
end


function Core.Tab_ApplyTemplate (frame, template, info, session)

	if template == "WHISPER" then
		local gender = (info[5] == 2 and "Male") or (info[5] == 3 and "Female")
		local _texture = string.format("Interface/CHARACTERFRAME/TemporaryPortrait-%s-%s", gender, info[4])

		frame.portrait:SetTexture(_texture)
		frame.sessionName:SetText(info[6])
		frame.sessionName:SetTextColor(GetClassColor(info[2]))
		frame.infoString:SetText(Core.utils.capitalize(info[2]))
		--frame.BG:SetGradient("Horizontal", 0,0,0, .28,.1,.25)
		
		
	elseif template == "BN_WHISPER" then
		frame.sessionName:SetTextColor(.6, .6, .8)
		frame.portrait:SetSize(40, 40)	
		frame.sessionName:SetText(info.accountName)
		
		Core.BNetDisplayUpdate(frame, 
			info.gameAccountInfo.characterName,
			info.gameAccountInfo.clientProgram,
			info.gameAccountInfo.isOnline
		)
		
		
	elseif template == "PARTY" then
		frame.portrait:SetTexture("Interface/COMMON/icon-".. UnitFactionGroup("player"))
		frame.sessionName:SetText(session.title)
		local c = ChatTypeInfo["PARTY"]
		frame.sessionName:SetTextColor(c.r, c.g, c.b)
		frame.infoString:SetText("")
		
		
	elseif template == "GUILD" then
		frame.portrait:SetTexture("Interface/GUILDFRAME/GuildLogo-NoLogo")
		frame.sessionName:SetText(session.title)
		local c = ChatTypeInfo["GUILD"]
		frame.sessionName:SetTextColor(c.r, c.g, c.b)
		local gn = GetGuildInfo("player")

		if #gn >= 12 then
			gn = gn:sub(1, 12).. "..."
		end
		
		frame.infoString:SetText(string.format("<%s>", gn))
		
		
	elseif template == "NEARBY" then
		frame.portrait:SetTexture("Interface/PVPFrame/Icons/PVP-Banner-Emblem-36")--"Interface/Worldmap/UI-World-Icon"
		frame.sessionName:SetText("Nearby")
		frame.infoString:SetText("Say & Emote")
		frame.sessionName:SetTextColor(.79, .95, 0)
		frame.BG:SetGradient("Horizontal", 0,0,0, .22, .22, 0)
		

	elseif template == "TEMP" then
		frame.portrait:SetTexture("Interface/Garrison/Portraits/FollowerPortrait_NoPortrait")
		frame.sessionName:SetText(Ambiguate(session.title, "short"))
		frame.sessionName:SetTextColor(1, 1, 1)
		frame.infoString:SetText("...")
	end
end


-------------------------------------------------------------------
function Core.CreateNewTab (info, session)
-------------------------------------------------------------------	
	local frame =  acquireTabFrame()

	Core.AppendTab(frame)
	frame.portrait:SetSize(48, 48)
	frame.BG:SetGradient("Horizontal", 0,0,0, .2,.2,.38)
	
	local template
	
	if session.needsInfo then
		template = "TEMP"
		
	elseif utils.any(session.chatType, "GUILD", "PARTY", "WHISPER", "BN_WHISPER") then
		template = session.chatType
		
	elseif session.name == "NEARBY" then
		template = "NEARBY"
	end
	
	Core.Tab_ApplyTemplate(frame, template, info, session)
	
	Core.orderedTabs[#Core.orderedTabs+1] = frame
	frame.tabIndex = #Core.orderedTabs
	
	Core.UI:PseudoScrollUpdate()
	
	return frame	
end






